import { useCallback, useEffect } from "react";
import { useNavigate } from "react-router-dom";
import useConfirmationDialog from "./ConfirmationDialog/useConfirmationDialog";
import { Cross1Icon } from "@radix-ui/react-icons";

interface ModalProps {
	modalId: string;
	title?: string;
	subTitle?: string;
	content?: string;
	navigatePathOnClose?: string;
	children?: React.ReactNode;
	customModalStyles?: string;
	customModalBoxStyles?: string;
}
/**
 * @description A modal component that can be used to display a fairly custom modal/dialog
 * @param modalId The id of the modal
 * @param title The title of the modal
 * @param subTitle The subtitle of the modal
 * @param content The content of the modal
 * @param navigatePathOnClose The path to navigate to when the modal is closed
 * @param children The children of the modal
 * @param customModalStyles Custom styles for the modal
 * @param customModalBoxStyles Custom styles for the modal box
 * @returns A modal component
 */
export const Modal = ({
	modalId,
	navigatePathOnClose,
	title,
	subTitle,
	content,
	children,
	customModalStyles,
	customModalBoxStyles,
}: ModalProps) => {
	const navigate = useNavigate();
	const { isConfirmationDialogVisible, hideDialog } = useConfirmationDialog();

	const handleClose = useCallback(() => {
		if (navigatePathOnClose) navigate(navigatePathOnClose);
		if (isConfirmationDialogVisible) (document.getElementById(modalId) as HTMLDialogElement).close();
		hideDialog();
	}, [hideDialog, isConfirmationDialogVisible, modalId, navigate, navigatePathOnClose]);

	const handleKeyPress = useCallback(
		(event: KeyboardEvent) => {
			if (event.key === "Escape") {
				handleClose();
			}
		},
		[handleClose],
	);
	// Handle modal open
	useEffect(() => {
		if (!modalId) return;
		(document.getElementById(modalId) as HTMLDialogElement).showModal();
		document.addEventListener("keydown", handleKeyPress);
		// Handle modal close
		return () => {
			document.removeEventListener("keydown", handleKeyPress);
			handleClose;
		};
	}, [handleClose, handleKeyPress, modalId]);

	const composeModalStyles = useCallback(() => {
		const _baseStyles = "modal modal-bottom sm:modal-middle w-screen z-50";
		if (customModalStyles) return _baseStyles + " " + customModalStyles;
		return _baseStyles;
	}, [customModalStyles]);
	const comopseModalBoxStyles = useCallback(() => {
		const _baseStyles = "modal-box";
		if (customModalBoxStyles) return _baseStyles + " " + customModalBoxStyles;
		return _baseStyles;
	}, [customModalBoxStyles]);

	return (
		<dialog id={modalId} className={composeModalStyles()}>
			<div className={comopseModalBoxStyles()}>
				<h3 className="font-bold text-lg">{title ?? "Hello!"}</h3>
				<p className="py-4">{subTitle ?? "Press ESC key or click the button below to close"}</p>
				<p>{content}</p>
				<div className="modal-action">
					<form method="dialog">
						<button className="btn btn-circle btn-sm absolute top-2 right-2" onClick={handleClose}>
							<Cross1Icon />
						</button>
						{children}
					</form>
				</div>
			</div>
		</dialog>
	);
};

export default Modal;
