/**
 * This file is used to configure react-router-dom.
 * Do not change anything here, unless you know what you are doing.
 * Every path and route is configured in `configure.tsx` file.
 */

import { createBrowserRouter, RouteObject } from "react-router-dom";
import { CustomRouteObject, navigation } from "./configure.tsx";
import Redirect from "./utils/Redirect.tsx";

/**
 * This function is used to convert `CustomRouteObject` to `RouteObject` and process all additionalProps
 */
const createRoutingFromCustomRouteObject = (routes: CustomRouteObject[]): RouteObject[] => {
	const navigationRoutes: RouteObject[] = []; //Root routes that will be used for navigation
	routes.forEach((route) => {
		// Extract everything except additionalProps and children
		const { additionalProps, children, ...routeNative } = route;
		//Filter routes that are disabled for navigation
		const isRouteDisabled = additionalProps.disableRedirect;
		if (isRouteDisabled) {
			return;
		}
		//Add route to navigationRoutes, array that will be returned
		const isRouteWithChildren = children !== undefined && children.length > 0;
		if (!isRouteWithChildren) {
			navigationRoutes.push({
				...routeNative,
			});
		} else {
			navigationRoutes.push({
				path: route.path,
				element: route.element,
				children: createRoutingFromCustomRouteObject(children),
			});
		}
	});
	//In case of empty routes, add a default route with message
	if (navigationRoutes.length === 0) {
		navigationRoutes.push({
			path: "/",
			element: <div>Empty routes</div>,
		});
	}
	//Always add redirect unmached route to root at the end
	return navigationRoutes;
};

/** It is main variable that handle all navigation rules and paths */
export const routes: RouteObject[] = [
	...createRoutingFromCustomRouteObject(navigation),
	{
		path: "*",
		element: <Redirect to="/" />,
	},
];

/** It is variable that have routes for react-router-dom, final form */
export const router = createBrowserRouter(routes, {
	basename: "/",
	future: {
		// Normalize `useNavigation()`/`useFetcher()` `formMethod` to uppercase
		v7_normalizeFormMethod: true,
	},
});

export default router;
